      SUBROUTINE cdfbet(which,p,x,a,b,status,bound)
C**********************************************************************
C
C      SUBROUTINE CDFBET( WHICH, P, X, A, B, STATUS, BOUND )
C               Cumulative Distribution Function
C                         BETa Distribution
C
C
C                              Function
C
C
C     Calculates any one parameter of the beta distribution given
C     values for the others.
C
C
C                              Arguments
C
C
C     WHICH --> Integer indicating which of the next four argument
C               values is to be calculated from the others.
C               Legal range: 1..4
C                    INTEGER WHICH
C
C     P <--> The integral from 0 to X of the chi-square
C            distribution.
C            Input range: [0, 1].
C                    REAL P
C
C     X <--> Upper limit of integration of beta density.
C            Input range: [0,1].
C            Search range: [0,1]
C                    REAL X
C
C     A <--> The first parameter of the beta density.
C            Input range: (0, +infinity).
C            Search range: [1E-30,1E30]
C                    REAL A
C
C     B <--> The second parameter of the beta density.
C            Input range: (0, +infinity).
C            Search range: [1E-30,1E30]
C                    REAL B
C
C     STATUS <-- 0 if calculation completed correctly
C               -I if input parameter number I is out of range
C                1 if answer appears to be lower than lowest
C                  search bound
C                2 if answer appears to be higher than greatest
C                  search bound
C                    INTEGER STATUS
C
C     BOUND <-- Undefined if STATUS is 0
C
C               Bound exceeded by parameter number I if STATUS
C               is negative.
C
C               Lower search bound if STATUS is 1.
C
C               Upper search bound if STATUS is 2.
C
C
C                              Method
C
C
C     Cumulative distribution function  (P)  is calculated directly by
C     code associated with the following reference.
C
C     DiDinato, A. R. and Morris,  A.   H.  Algorithm 708: Significant
C     Digit Computation of the Incomplete  Beta  Function Ratios.  ACM
C     Trans. Math.  Softw. 18 (1993), 360-373.
C
C     Computation of other parameters involve a seach for a value that
C     produces  the desired  value  of P.   The search relies  on  the
C     monotinicity of P with the other parameter.
C
C
C                              Note
C
C
C     The beta density is proportional to
C               t^(A-1) * (1-t)^(B-1)
C
C**********************************************************************
C     .. Parameters ..
      REAL tol
      PARAMETER (tol=1.0E-4)
      REAL zero,inf
      PARAMETER (zero=1.0E-30,inf=1.0E30)
C     ..
C     .. Scalar Arguments ..
      REAL a,b,bound,p,x
      INTEGER status,which
C     ..
C     .. Local Scalars ..
      REAL aa,bb,fx,pp,xhi,xlo,zz
      LOGICAL qhi,qleft
C     ..
C     .. External Functions ..
      REAL cumbet
      EXTERNAL cumbet
C     ..
C     .. External Subroutines ..
      EXTERNAL invr,stinvr,stzror,zror
C     ..
C     .. Statement Functions ..
      REAL cum
C     ..
C     .. Statement Function definitions ..
      cum(zz,aa,bb,pp) = cumbet(zz,aa,bb) - pp
C     ..
C     .. Executable Statements ..
C
C     Check arguments
C
      IF (.NOT. ((which.LT.1).OR. (which.GT.4))) GO TO 10
      IF (which.LT.1) THEN
          bound = 1.0

      ELSE
          bound = 4.0
      END IF

      status = -1
      RETURN

   10 IF (which.EQ.1) GO TO 30
C
C     P
C
      IF (.NOT. ((p.LT.0.0).OR. (p.GT.1))) GO TO 20
      IF (p.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = 1.0
      END IF

      status = -2
      RETURN

   20 CONTINUE
   30 IF (which.EQ.2) GO TO 50
C
C     X
C
      IF (.NOT. ((x.LT.0.0).OR. (x.GT.1))) GO TO 40
      IF (x.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = 1.0
      END IF

      status = -3
      RETURN

   40 CONTINUE
   50 IF (which.EQ.3) GO TO 70
C
C     A
C
      IF (.NOT. (a.LE.0.0)) GO TO 60
      bound = 0.0
      status = -4
      RETURN

   60 CONTINUE
   70 IF (which.EQ.4) GO TO 90
C
C     B
C
      IF (.NOT. (b.LE.0.0)) GO TO 80
      bound = 0.0
      status = -5
      RETURN

   80 CONTINUE
C
C     Calculate ANSWERS
C
   90 IF ((1).EQ. (which)) THEN
C
C     Calculating P
C
          p = cumbet(x,a,b)
          status = 0

      ELSE IF ((2).EQ. (which)) THEN
C
C     Calculating X
C
          CALL stzror(0.0,1.0,tol,tol)
          status = 0
          CALL zror(status,x,fx,xlo,xhi,qleft,qhi)
  100     IF (.NOT. (status.EQ.1)) GO TO 110
          fx = cum(x,a,b,p)
          CALL zror(status,x,fx,xlo,xhi,qleft,qhi)
          GO TO 100

  110     IF (.NOT. (status.EQ.-1)) GO TO 140
          IF (.NOT. (qleft)) GO TO 120
          status = 1
          bound = 0.0
          GO TO 130

  120     status = 2
          bound = 1.0
  130     CONTINUE
  140     CONTINUE

      ELSE IF ((3).EQ. (which)) THEN
C
C     Computing A
C
          a = 5.0
          CALL stinvr(zero,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,a,fx,qleft,qhi)
  150     IF (.NOT. (status.EQ.1)) GO TO 160
          fx = cum(x,a,b,p)
          CALL invr(status,a,fx,qleft,qhi)
          GO TO 150

  160     IF (.NOT. (status.EQ.-1)) GO TO 190
          IF (.NOT. (qleft)) GO TO 170
          status = 1
          bound = zero
          GO TO 180

  170     status = 2
          bound = inf
  180     CONTINUE
  190     CONTINUE

      ELSE IF ((4).EQ. (which)) THEN
C
C     Computing B
C
          b = 5.0
          CALL stinvr(zero,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,b,fx,qleft,qhi)
  200     IF (.NOT. (status.EQ.1)) GO TO 210
          fx = cum(x,a,b,p)
          CALL invr(status,b,fx,qleft,qhi)
          GO TO 200

  210     IF (.NOT. (status.EQ.-1)) GO TO 240
          IF (.NOT. (qleft)) GO TO 220
          status = 1
          bound = zero
          GO TO 230

  220     status = 2
          bound = inf
  230     CONTINUE
  240 END IF

      RETURN

      END
