      SUBROUTINE cdfbin(which,p,s,xn,pr,status,bound)
C**********************************************************************
C
C      SUBROUTINE CDFBIN ( WHICH, P, S, XN, PR, STATUS, BOUND )
C               Cumulative Distribution Function
C                         BINomial distribution
C
C
C                              Function
C
C
C     Calculates any one parameter of the binomial
C     distribution given values for the others.
C
C
C                              Arguments
C
C
C     WHICH --> Integer indicating which of the next four argument
C               values is to be calculated from the others.
C               Legal range: 1..4
C                    INTEGER WHICH
C
C     P <--> The cumulation from 0 to S of the binomial distribution.
C            (Probablility of S or fewer successes in XN trials each
C            with probability of success PR.)
C            Input range: [0,1].
C                    REAL P
C
C     S <--> The number of successes observed.
C            Input range: [0, XN]
C            Search range: [0, XN]
C                    REAL S
C
C     XN  <--> The number of binomial trials.
C              Input range: (0, +infinity).
C              Search range: [1E-30, 1E30]
C                    REAL XN
C
C     PR  <--> The probability of success in each binomial trial.
C              Input range: [0,1].
C              Search range: [0,1]
C                    REAL PR
C
C     STATUS <-- 0 if calculation completed correctly
C               -I if input parameter number I is out of range
C                1 if answer appears to be lower than lowest
C                  search bound
C                2 if answer appears to be higher than greatest
C                  search bound
C                    INTEGER STATUS
C
C     BOUND <-- Undefined if STATUS is 0
C
C               Bound exceeded by parameter number I if STATUS
C               is negative.
C
C               Lower search bound if STATUS is 1.
C
C               Upper search bound if STATUS is 2.
C
C
C                              Method
C
C
C     Formula  26.5.24    of   Abramowitz  and    Stegun,  Handbook   of
C     Mathematical   Functions (1966) is   used  to reduce the  binomial
C     distribution  to  the  cumulative incomplete    beta distribution.
C
C     Computation of other parameters involve a seach for a value that
C     produces  the desired  value  of P.   The search relies  on  the
C     monotinicity of P with the other parameter.
C
C
C**********************************************************************
C     .. Parameters ..
      REAL tol
      PARAMETER (tol=1.0E-4)
      REAL zero,inf
      PARAMETER (zero=1.0E-30,inf=1.0E30)
C     ..
C     .. Scalar Arguments ..
      REAL bound,p,pr,s,xn
      INTEGER status,which
C     ..
C     .. Local Scalars ..
      REAL fx,pp,ppr,ss,xhi,xlo,xnxn
      LOGICAL qhi,qleft
C     ..
C     .. External Functions ..
      REAL cumbin
      EXTERNAL cumbin
C     ..
C     .. External Subroutines ..
      EXTERNAL invr,stinvr,stzror,zror
C     ..
C     .. Statement Functions ..
      REAL cum
C     ..
C     .. Statement Function definitions ..
      cum(ss,xnxn,ppr,pp) = cumbin(ss,xnxn,ppr) - pp
C     ..
C     .. Executable Statements ..
C
C     Check arguments
C
      IF (.NOT. ((which.LT.1).AND. (which.GT.4))) GO TO 10
      IF (which.LT.1) THEN
          bound = 1.0

      ELSE
          bound = 4.0
      END IF

      status = -1
      RETURN

   10 IF (which.EQ.1) GO TO 30
C
C     P
C
      IF (.NOT. ((p.LT.0.0).OR. (p.GT.1.0))) GO TO 20
      IF (p.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = 1.0
      END IF

      status = -2
      RETURN

   20 CONTINUE
   30 IF (which.EQ.3) GO TO 50
C
C     XN
C
      IF (.NOT. (xn.LE.0.0)) GO TO 40
      bound = 0.0
      status = -4
      RETURN

   40 CONTINUE
   50 IF (which.EQ.2) GO TO 70
C
C     S
C
      IF (.NOT. ((s.LT.0.0).OR. ((which.NE.3).AND.
     +    (s.GT.xn)))) GO TO 60
      IF (s.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = xn
      END IF

      status = -3
      RETURN

   60 CONTINUE
   70 IF (which.EQ.4) GO TO 90
C
C     PR
C
      IF (.NOT. ((pr.LT.0.0).OR. (pr.GT.1.0))) GO TO 80
      IF (pr.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = 1.0
      END IF

      status = -5
      RETURN

   80 CONTINUE
C
C     Calculate ANSWERS
C
   90 IF ((1).EQ. (which)) THEN
C
C     Calculating P
C
          p = cumbin(s,xn,pr)
          status = 0

      ELSE IF ((2).EQ. (which)) THEN
C
C     Calculating S
C
          s = 5.0
          CALL stinvr(0.0,xn,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,s,fx,qleft,qhi)
  100     IF (.NOT. (status.EQ.1)) GO TO 110
          fx = cum(s,xn,pr,p)
          CALL invr(status,s,fx,qleft,qhi)
          GO TO 100

  110     IF (.NOT. (status.EQ.-1)) GO TO 140
          IF (.NOT. (qleft)) GO TO 120
          status = 1
          bound = 0.0
          GO TO 130

  120     status = 2
          bound = xn
  130     CONTINUE
  140     CONTINUE

      ELSE IF ((3).EQ. (which)) THEN
C
C     Calculating XN
C
          xn = 5.0
          CALL stinvr(zero,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,xn,fx,qleft,qhi)
  150     IF (.NOT. (status.EQ.1)) GO TO 160
          fx = cum(s,xn,pr,p)
          CALL invr(status,xn,fx,qleft,qhi)
          GO TO 150

  160     IF (.NOT. (status.EQ.-1)) GO TO 190
          IF (.NOT. (qleft)) GO TO 170
          status = 1
          bound = zero
          GO TO 180

  170     status = 2
          bound = inf
  180     CONTINUE
  190     CONTINUE

      ELSE IF ((4).EQ. (which)) THEN
C
C     Calculating PR
C
          CALL stzror(0.0,1.0,tol,tol)
          status = 0
          CALL zror(status,pr,fx,xlo,xhi,qleft,qhi)
  200     IF (.NOT. (status.EQ.1)) GO TO 210
          fx = cum(s,xn,pr,p)
          CALL zror(status,pr,fx,xlo,xhi,qleft,qhi)
          GO TO 200

  210     IF (.NOT. (status.EQ.-1)) GO TO 240
          IF (.NOT. (qleft)) GO TO 220
          status = 1
          bound = 0.0
          GO TO 230

  220     status = 2
          bound = 1.0
  230     CONTINUE
  240 END IF

      RETURN

      END
