      SUBROUTINE cdff(which,p,f,dfn,dfd,status,bound)
C**********************************************************************
C
C      SUBROUTINE CDFF( WHICH, P, F, DFN, DFD, STATUS, BOUND )
C               Cumulative Distribution Function
C               F distribution
C
C
C                              Function
C
C
C     Calculates any one parameter of the F distribution
C     given values for the others.
C
C
C                              Arguments
C
C
C     WHICH --> Integer indicating which of the next four argument
C               values is to be calculated from the others.
C               Legal range: 1..4
C                    INTEGER WHICH
C
C       P <--> The integral from 0 to F of the f-density.
C              Input range: [0,1-1E-7].
C                    REAL P
C
C       F <--> Upper limit of integration of the f-density.
C              Input range: [0, +infinity).
C              Search range: [0,1E30]
C                    REAL F
C
C     DFN < --> Degrees of freedom of the numerator sum of squares.
C               Input range: (0, +infinity).
C               Search range: [ 1E-30, 1E30]
C                    REAL DFN
C
C     DFD < --> Degrees of freedom of the denominator sum of squares.
C               Input range: (0, +infinity).
C               Search range: [ 1E-30, 1E30]
C                    REAL DFD
C
C     STATUS <-- 0 if calculation completed correctly
C               -I if input parameter number I is out of range
C                1 if answer appears to be lower than lowest
C                  search bound
C                2 if answer appears to be higher than greatest
C                  search bound
C                    INTEGER STATUS
C
C     BOUND <-- Undefined if STATUS is 0
C
C               Bound exceeded by parameter number I if STATUS
C               is negative.
C
C               Lower search bound if STATUS is 1.
C
C               Upper search bound if STATUS is 2.
C
C
C                              Method
C
C
C     Formula   26.6.2   of   Abramowitz   and   Stegun,  Handbook  of
C     Mathematical  Functions (1966) is used to reduce the computation
C     of the  cumulative  distribution function for the  F  variate to
C     that of an incomplete beta.
C
C     Computation of other parameters involve a seach for a value that
C     produces  the desired  value  of P.   The search relies  on  the
C     monotinicity of P with the other parameter.
C
C                              WARNING
C
C     The value of the  cumulative  F distribution is  not necessarily
C     monotone in  either degrees of freedom.  There  thus may  be two
C     values  that  provide a given CDF  value.   This routine assumes
C     monotonicity and will find an arbitrary one of the two values.
C
C**********************************************************************
C     .. Parameters ..
      REAL tol
      PARAMETER (tol=1.0E-4)
      REAL zero,one,inf
      PARAMETER (zero=1.0E-30,one=1.0-1.0E-7,inf=1.0E30)
C     ..
C     .. Scalar Arguments ..
      REAL bound,dfd,dfn,f,p
      INTEGER status,which
C     ..
C     .. Local Scalars ..
      REAL dfdd,dfnn,ff,fx,pp
      LOGICAL qhi,qleft
C     ..
C     .. External Functions ..
      REAL cumf
      EXTERNAL cumf
C     ..
C     .. External Subroutines ..
      EXTERNAL invr,stinvr
C     ..
C     .. Statement Functions ..
      REAL cum
C     ..
C     .. Statement Function definitions ..
      cum(ff,dfnn,dfdd,pp) = cumf(ff,dfnn,dfdd) - pp
C     ..
C     .. Executable Statements ..
C
C     Check arguments
C
      IF (.NOT. ((which.LT.1).OR. (which.GT.4))) GO TO 10
      IF (which.LT.1) THEN
          bound = 1.0

      ELSE
          bound = 4.0
      END IF

      status = -1
      RETURN

   10 IF (which.EQ.1) GO TO 30
C
C     P
C
      IF (.NOT. ((p.LT.0.0).OR. (p.GT.one))) GO TO 20
      IF (p.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = one
      END IF

      status = -2
      RETURN

   20 CONTINUE
   30 IF (which.EQ.2) GO TO 50
C
C     F
C
      IF (.NOT. (f.LT.0.0)) GO TO 40
      bound = 0.0
      status = -3
      RETURN

   40 CONTINUE
   50 IF (which.EQ.3) GO TO 70
C
C     DFN
C
      IF (.NOT. (dfn.LE.0.0)) GO TO 60
      bound = 0.0
      status = -4
      RETURN

   60 CONTINUE
   70 IF (which.EQ.4) GO TO 90
C
C     DFD
C
      IF (.NOT. (dfd.LE.0.0)) GO TO 80
      bound = 0.0
      status = -5
      RETURN

   80 CONTINUE
C
C     Calculate ANSWERS
C
   90 IF ((1).EQ. (which)) THEN
C
C     Calculating P
C
          p = cumf(f,dfn,dfd)
          status = 0

      ELSE IF ((2).EQ. (which)) THEN
C
C     Calculating F
C
          f = 5.0
          CALL stinvr(0.0,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,f,fx,qleft,qhi)
  100     IF (.NOT. (status.EQ.1)) GO TO 110
          fx = cum(f,dfn,dfd,p)
          CALL invr(status,f,fx,qleft,qhi)
          GO TO 100

  110     IF (.NOT. (status.EQ.-1)) GO TO 140
          IF (.NOT. (qleft)) GO TO 120
          status = 1
          bound = 0.0
          GO TO 130

  120     status = 2
          bound = inf
  130     CONTINUE
  140     CONTINUE

      ELSE IF ((3).EQ. (which)) THEN
C
C     Calculating DFN
C
          dfn = 5.0
          CALL stinvr(zero,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,dfn,fx,qleft,qhi)
  150     IF (.NOT. (status.EQ.1)) GO TO 160
          fx = cum(f,dfn,dfd,p)
          CALL invr(status,dfn,fx,qleft,qhi)
          GO TO 150

  160     IF (.NOT. (status.EQ.-1)) GO TO 190
          IF (.NOT. (qleft)) GO TO 170
          status = 1
          bound = zero
          GO TO 180

  170     status = 2
          bound = inf
  180     CONTINUE
  190     CONTINUE

      ELSE IF ((4).EQ. (which)) THEN
C
C     Calculating DFD
C
          dfd = 5.0
          CALL stinvr(zero,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,dfd,fx,qleft,qhi)
  200     IF (.NOT. (status.EQ.1)) GO TO 210
          fx = cum(f,dfn,dfd,p)
          CALL invr(status,dfd,fx,qleft,qhi)
          GO TO 200

  210     IF (.NOT. (status.EQ.-1)) GO TO 240
          IF (.NOT. (qleft)) GO TO 220
          status = 1
          bound = zero
          GO TO 230

  220     status = 2
          bound = inf
  230     CONTINUE
  240 END IF

      RETURN

      END
