      SUBROUTINE cdfgam(which,p,x,shape,scale,status,bound)
C**********************************************************************
C
C      SUBROUTINE CDFGAM( WHICH, P, X, SHAPE, SCALE, STATUS, BOUND )
C               Cumulative Distribution Function
C                         GAMma Distribution
C
C
C                              Function
C
C
C     Calculates any one parameter of the gamma
C     distribution given values for the others.
C
C
C                              Arguments
C
C
C     WHICH --> Integer indicating which of the next four argument
C               values is to be calculated from the others.
C               Legal range: 1..4
C                    INTEGER WHICH
C
C     P <--> The integral from 0 to X of the gamma density.
C            Input range: [0,1-1E-7].
C		     Real*8 P
C
C     X <--> The upper limit of integration of the gamma density.
C            Input range: [0, +infinity).
C            Search range: [0,1E30]
C		     Real*8 X
C
C     SHAPE <--> The shape parameter of the gamma density.
C                Input range: (0, +infinity).
C                Search range: [1E-30,1E30]
C		   Real*8 SHAPE
C
C
C     SCALE <--> The scale parameter of the gamma density.
C                Input range: (0, +infinity).
C                Search range: (1E-30,1E30]
C		    Real*8 SCALE
C
C     STATUS <-- 0 if calculation completed correctly
C               -I if input parameter number I is out of range
C                1 if answer appears to be lower than lowest
C                  search bound
C                2 if answer appears to be higher than greatest
C                  search bound
C                10 if the gamma or inverse gamma routine cannot
C                   compute the answer.  Usually happens only for
C                   X and SHAPE very large (gt 1E10 or more)
C                    INTEGER STATUS
C
C     BOUND <-- Undefined if STATUS is 0
C
C               Bound exceeded by parameter number I if STATUS
C               is negative.
C
C               Lower search bound if STATUS is 1.
C
C               Upper search bound if STATUS is 2.
C
C
C                              Method
C
C
C     Cumulative distribution function (P) is calculated directly by
C     the code associated with:
C
C     DiDinato, A. R. and Morris, A. H. Computation of the  incomplete
C     gamma function  ratios  and their  inverse.   ACM  Trans.  Math.
C     Softw. 12 (1986), 377-393.
C
C     Computation of other parameters involve a seach for a value that
C     produces  the desired  value  of P.   The search relies  on  the
C     monotinicity of P with the other parameter.
C
C
C                              Note
C
C
C
C     The gamma density is proportional to
C       T**(SHAPE - 1) * EXP(- SCALE * T)
C
C
C**********************************************************************
C     .. Parameters ..
      Real*8 tol
      PARAMETER (tol=1.0E-4)
      REAL zero,one,inf
      PARAMETER (zero=1.0E-30,one=1.0-1.0E-7,inf=1.0E30)
C     ..
C     .. Scalar Arguments ..
      REAL bound,p,scale,shape,x
      INTEGER status,which
C     ..
C     .. Local Scalars ..
      DOUBLE PRECISION xx
      REAL aa,fx,pp,xscale,zz
      INTEGER ierr
      LOGICAL qhi,qleft
C     ..
C     .. External Functions ..
      REAL cumgam
      EXTERNAL cumgam
C     ..
C     .. External Subroutines ..
      EXTERNAL gaminv,invr,stinvr
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC dble
C     ..
C     .. Statement Functions ..
      REAL cum
C     ..
C     .. Statement Function definitions ..
      cum(zz,aa,pp) = cumgam(zz,aa) - pp
C     ..
C     .. Executable Statements ..
C
C     Check arguments
C
      IF (.NOT. ((which.LT.1).OR. (which.GT.4))) GO TO 10
      IF (which.LT.1) THEN
          bound = 1.0

      ELSE
          bound = 4.0
      END IF

      status = -1
      RETURN

   10 IF (which.EQ.1) GO TO 30
C
C     P
C
      IF (.NOT. ((p.LT.0.0).OR. (p.GT.one))) GO TO 20
      IF (p.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = one
      END IF

      status = -2
      RETURN

   20 CONTINUE
   30 IF (which.EQ.2) GO TO 50
C
C     X
C
      IF (.NOT. (x.LT.0.0)) GO TO 40
      bound = 0.0
      status = -3
      RETURN

   40 CONTINUE
   50 IF (which.EQ.3) GO TO 70
C
C     SHAPE
C
      IF (.NOT. (shape.LE.0.0)) GO TO 60
      bound = 0.0
      status = -4
      RETURN

   60 CONTINUE
   70 IF (which.EQ.4) GO TO 90
C
C     SCALE
C
      IF (.NOT. (scale.LE.0.0)) GO TO 80
      bound = 0.0
      status = -5
      RETURN

   80 CONTINUE
C
C     Calculate ANSWERS
C
   90 IF ((1).EQ. (which)) THEN
C
C     Calculating P
C
          status = 0
          xscale = x*scale
          p = cumgam(xscale,shape)
          IF (p.GT.1.5) status = 10

      ELSE IF ((2).EQ. (which)) THEN
C
C     Computing X
C
          CALL gaminv(dble(shape),xx,-1.0D0,dble(p),dble(1.0D0-p),ierr)
          IF (ierr.LT.0.0) THEN
              status = 10
              RETURN

          ELSE
              x = xx/scale
              status = 0
          END IF

      ELSE IF ((3).EQ. (which)) THEN
C
C     Computing SHAPE
C
          shape = 5.0
          xscale = x*scale
          CALL stinvr(zero,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,shape,fx,qleft,qhi)
  100     IF (.NOT. (status.EQ.1)) GO TO 110
          fx = cum(xscale,shape,p)
          IF ((fx+p).GT.1.5) THEN
              status = 10
              RETURN

          END IF

          CALL invr(status,shape,fx,qleft,qhi)
          GO TO 100

  110     IF (.NOT. (status.EQ.-1)) GO TO 140
          IF (.NOT. (qleft)) GO TO 120
          status = 1
          bound = zero
          GO TO 130

  120     status = 2
          bound = inf
  130     CONTINUE
  140     CONTINUE

      ELSE IF ((4).EQ. (which)) THEN
C
C     Computing SCALE
C
          CALL gaminv(dble(shape),xx,-1.0D0,dble(p),dble(1.0D0-p),ierr)
          IF (ierr.LT.0.0) THEN
              status = 10
              RETURN

          ELSE
              scale = xx/x
              status = 0
          END IF

      END IF

      RETURN

      END
