      SUBROUTINE cdfnor(which,p,x,mean,sd,status,bound)
C**********************************************************************
C
C      SUBROUTINE CDFNOR( WHICH, P, X, MEAN, SD, STATUS, BOUND )
C               Cumulative Distribution Function
C               NORmal distribution
C
C
C                              Function
C
C
C     Calculates any one parameter of the normal
C     distribution given values for the others.
C
C
C                              Arguments
C
C
C     WHICH  --> Integer indicating  which of the  next  parameter
C     values is to be calculated using values  of the others.
C     Legal range: 1..4
C                    INTEGER WHICH
C
C     P <--> The integral from -infinity to X of the normal density.
C            Input range: [1E-7,1-1E-7].
C		     Real*8 P
C
C     X < --> Upper limit of integration of the normal-density.
C             Input range: ( -infinity, +infinity)
C		     Real*8 X
C
C     MEAN <--> The mean of the normal density.
C               Input range: (-infinity, +infinity)
C		     Real*8 MEAN
C
C     SD <--> Standard Deviation of the normal density.
C             Input range: (0, +infinity).
C		     Real*8 SD
C
C     STATUS <-- 0 if calculation completed correctly
C               -I if input parameter number I is out of range
C                1 if answer appears to be lower than lowest
C                  search bound
C                2 if answer appears to be higher than greatest
C                  search bound
C                    INTEGER STATUS
C
C     BOUND <-- Undefined if STATUS is 0
C
C               Bound exceeded by parameter number I if STATUS
C               is negative.
C
C               Lower search bound if STATUS is 1.
C
C               Upper search bound if STATUS is 2.
C
C
C                              Method
C
C
C     The rational functions from pages  90-95  of Kennedy and Gentle,
C     Statistical  Computing,  Marcel  Dekker, NY,  1980 are  used  to
C     compute  both  the  cumulative  and   inverse  standard  normal.
C     Therefore no searches are necessary for any parameter.
C
C     For X < -15, the asymptotic expansion for the normal is used.
C     This is formula 26.2.12 of Abramowitz and Stegun.
C
C
C                              Note
C
C
C      The normal density is proportional to
C      exp( - 0.5 * (( X - MEAN)/SD)**2)
C
C
C**********************************************************************
C     .. Parameters ..
      Real*8 zero,one
      PARAMETER (zero=1.0D-15,one=1.0D0-1.0E-15)
C     ..
C     .. Scalar Arguments ..
      Real*8 bound,mean,p,sd,x
      INTEGER status,which
C     ..
C     .. Local Scalars ..
      Real*8 z
C     ..
C     .. External Functions ..

      Real*8 cumnor,invnor
      EXTERNAL cumnor,invnor
C     ..
C     .. Executable Statements ..
C
C     Check arguments
C
      status = 0
      IF (.NOT. ((which.LT.1).OR. (which.GT.4))) GO TO 10
      IF (which.LT.1) THEN
          bound = 1.0

      ELSE
          bound = 4.0
      END IF

      status = -1
      RETURN

   10 IF (which.EQ.1) GO TO 30
C
C     P
C
      IF (.NOT. ((p.LT.zero).OR. (p.GT.one))) GO TO 20
      IF (p.LT.zero) THEN
          bound = zero

      ELSE
          bound = one
      END IF

      status = -2
      RETURN

   20 CONTINUE
   30 IF (which.EQ.4) GO TO 50
C
C     SD
C
      IF (.NOT. (sd.LE.0.0)) GO TO 40
      bound = 0.0
      status = -5
      RETURN

   40 CONTINUE
C
C     Calculate ANSWERS
C
   50 IF ((1).EQ. (which)) THEN
C
C     Computing P
C
          z = (x-mean)/sd
	  p = cumnor(z)

      ELSE IF ((2).EQ. (which)) THEN
C
C     Computing X
C
	  z = invnor(p)
          x = sd*z + mean

      ELSE IF ((3).EQ. (which)) THEN
C
C     Computing the MEAN
C
          z = invnor(p)
          mean = x - sd*z

      ELSE IF ((4).EQ. (which)) THEN
C
C     Computing SD
C
          z = invnor(p)
          sd = (x-mean)/z
      END IF

      RETURN

      END
