      SUBROUTINE cdfpoi(which,p,s,xlam,status,bound)
C**********************************************************************
C
C      SUBROUTINE CDFPOI( WHICH, P, S, XLAM, STATUS, BOUND )
C               Cumulative Distribution Function
C               POIsson distribution
C
C
C                              Function
C
C
C     Calculates any one parameter of the Poisson
C     distribution given values for the others.
C
C
C                              Arguments
C
C
C     WHICH --> Integer indicating which  argument
C               value is to be calculated from the others.
C               Legal range: 1..3
C                    INTEGER WHICH
C
C        P <--> The cumulation from 0 to S of the poisson density.
C               Input range: [0,1-1E-7].
C                    REAL P
C
C        S <--> Upper limit of cumulation of the Poisson.
C               Input range: [0, +infinity).
C               Search range: [0,1E30]
C               REAL S
C
C     XLAM <--> Mean of the Poisson distribution.
C               Input range: [0, +infinity).
C               Search range: [0,1E30]
C               REAL XLAM
C
C
C                              Method
C
C
C     Formula   26.4.21  of   Abramowitz  and   Stegun,   Handbook  of
C     Mathematical Functions (1966) is used  to reduce the computation
C     of  the cumulative distribution function to that  of computing a
C     chi-square, hence an incomplete gamma function.
C
C     Cumulative  distribution function  (P) is  calculated  directly.
C     Computation of other parameters involve a seach for a value that
C     produces  the desired value of  P.   The  search relies  on  the
C     monotinicity of P with the other parameter.
C
C
C**********************************************************************
C     .. Parameters ..
      REAL tol
      PARAMETER (tol=1.0E-4)
      REAL one,inf
      PARAMETER (one=1.0-1.0E-7,inf=1.0E30)
C     ..
C     .. Scalar Arguments ..
      REAL bound,p,s,xlam
      INTEGER status,which
C     ..
C     .. Local Scalars ..
      REAL fx,pp,ss,xxlam
      LOGICAL qhi,qleft
C     ..
C     .. External Functions ..
      REAL cumpoi
      EXTERNAL cumpoi
C     ..
C     .. External Subroutines ..
      EXTERNAL invr,stinvr
C     ..
C     .. Statement Functions ..
      REAL cum
C     ..
C     .. Statement Function definitions ..
      cum(ss,xxlam,pp) = cumpoi(ss,xxlam) - pp
C     ..
C     .. Executable Statements ..
C
C     Check arguments
C
      IF (.NOT. ((which.LT.1).OR. (which.GT.3))) GO TO 10
      IF (which.LT.1) THEN
          bound = 1.0

      ELSE
          bound = 3.0
      END IF

      status = -1
      RETURN

   10 IF (which.EQ.1) GO TO 30
C
C     P
C
      IF (.NOT. ((p.LT.0.0).OR. (p.GT.one))) GO TO 20
      IF (p.LT.0.0) THEN
          bound = 0.0

      ELSE
          bound = one
      END IF

      status = -2
      RETURN

   20 CONTINUE
   30 IF (which.EQ.2) GO TO 50
C
C     S
C
      IF (.NOT. (s.LT.0.0)) GO TO 40
      bound = 0.0
      status = -3
      RETURN

   40 CONTINUE
   50 IF (which.EQ.3) GO TO 70
C
C     XLAM
C
      IF (.NOT. (xlam.LT.0.0)) GO TO 60
      bound = 0.0
      status = -4
      RETURN

   60 CONTINUE
C
C     Calculate ANSWERS
C
   70 IF ((1).EQ. (which)) THEN
C
C     Calculating P
C
          p = cumpoi(s,xlam)
          status = 0

      ELSE IF ((2).EQ. (which)) THEN
C
C     Calculating S
C
          s = 5.0
          CALL stinvr(0.0,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,s,fx,qleft,qhi)
   80     IF (.NOT. (status.EQ.1)) GO TO 90
          fx = cum(s,xlam,p)
          CALL invr(status,s,fx,qleft,qhi)
          GO TO 80

   90     IF (.NOT. (status.EQ.-1)) GO TO 120
          IF (.NOT. (qleft)) GO TO 100
          status = 1
          bound = 0.0
          GO TO 110

  100     status = 2
          bound = inf
  110     CONTINUE
  120     CONTINUE

      ELSE IF ((3).EQ. (which)) THEN
C
C     Calculating XLAM
C
          xlam = 5.0
          CALL stinvr(0.0,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,xlam,fx,qleft,qhi)
  130     IF (.NOT. (status.EQ.1)) GO TO 140
          fx = cum(s,xlam,p)
          CALL invr(status,xlam,fx,qleft,qhi)
          GO TO 130

  140     IF (.NOT. (status.EQ.-1)) GO TO 170
          IF (.NOT. (qleft)) GO TO 150
          status = 1
          bound = 0.0
          GO TO 160

  150     status = 2
          bound = inf
  160     CONTINUE
  170 END IF

      RETURN

      END
