      SUBROUTINE cdft(which,p,t,df,status,bound)
C**********************************************************************
C
C      SUBROUTINE CDFT( WHICH, P, T, DF, STATUS, BOUND )
C               Cumulative Distribution Function
C                         T distribution
C
C
C                              Function
C
C
C     Calculates any one parameter of the t distribution given
C     values for the others.
C
C
C                              Arguments
C
C
C     WHICH --> Integer indicating which  argument
C               values is to be calculated from the others.
C               Legal range: 1..3
C                    INTEGER WHICH
C
C        P <--> The integral from -infinity to t of the t-density.
C              Input range: [1E-7,1-1E-7].
C                    REAL P
C
C        T <--> Upper limit of integration of the t-density.
C               Input range: ( -infinity, +infinity).
C               Search range: [ -1E30, 1E30 ]
C                    REAL T
C
C        DF <--> Degrees of freedom of the t-distribution.
C                Input range: (0 , +infinity).
C                Search range: [1e-30, 1E30]
C                    REAL DF
C
C     STATUS <-- 0 if calculation completed correctly
C               -I if input parameter number I is out of range
C                1 if answer appears to be lower than lowest
C                  search bound
C                2 if answer appears to be higher than greatest
C                  search bound
C                    INTEGER STATUS
C
C     BOUND <-- Undefined if STATUS is 0
C
C               Bound exceeded by parameter number I if STATUS
C               is negative.
C
C               Lower search bound if STATUS is 1.
C
C               Upper search bound if STATUS is 2.
C
C
C                              Method
C
C
C     Formula  26.5.27  of   Abramowitz   and  Stegun,   Handbook   of
C     Mathematical Functions  (1966) is used to reduce the computation
C     of the cumulative distribution function to that of an incomplete
C     beta.
C
C     Computation of other parameters involve a seach for a value that
C     produces  the desired  value  of P.   The search relies  on  the
C     monotinicity of P with the other parameter.
C
C**********************************************************************
C     .. Parameters ..
      REAL tol
      PARAMETER (tol=1.0E-4)
      REAL zero,one,inf
      PARAMETER (zero=1.0E-30,one=1.0-1.0E-7,inf=1.0E30)
C     ..
C     .. Scalar Arguments ..
      REAL bound,df,p,t
      INTEGER status,which
C     ..
C     .. Local Scalars ..
      REAL dfdf,fx,pp,tt
      LOGICAL qhi,qleft
C     ..
C     .. External Functions ..
      REAL cumt,t1
      EXTERNAL cumt,t1
C     ..
C     .. External Subroutines ..
      EXTERNAL invr,stinvr
C     ..
C     .. Statement Functions ..
      REAL cum
C     ..
C     .. Statement Function definitions ..
      cum(tt,dfdf,pp) = cumt(tt,dfdf) - pp
C     ..
C     .. Executable Statements ..
C
C     Check arguments
C
      IF (.NOT. ((which.LT.1).OR. (which.GT.3))) GO TO 10
      IF (which.LT.1) THEN
          bound = 1.0

      ELSE
          bound = 3.0
      END IF

      status = -1
      RETURN

   10 IF (which.EQ.1) GO TO 30
C
C     P
C
      IF (.NOT. ((p.LT.zero).OR. (p.GT.one))) GO TO 20
      IF (p.LT.zero) THEN
          bound = zero

      ELSE
          bound = one
      END IF

      status = -2
      RETURN

   20 CONTINUE
   30 IF (which.EQ.3) GO TO 50
C
C     DF
C
      IF (.NOT. (df.LE.0.0)) GO TO 40
      bound = 0.0
      status = -4
      RETURN

   40 CONTINUE
C
C     Calculate ANSWERS
C
   50 IF ((1).EQ. (which)) THEN
C
C     Computing P
C
          p = cumt(t,df)
          status = 0

      ELSE IF ((2).EQ. (which)) THEN
C
C     Computing T
C
C     .. Get initial approximation for T
C
          t = t1(p,df)
          CALL stinvr(-inf,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,t,fx,qleft,qhi)
   60     IF (.NOT. (status.EQ.1)) GO TO 70
          fx = cum(t,df,p)
          CALL invr(status,t,fx,qleft,qhi)
          GO TO 60

   70     IF (.NOT. (status.EQ.-1)) GO TO 100
          IF (.NOT. (qleft)) GO TO 80
          status = 1
          bound = -inf
          GO TO 90

   80     status = 2
          bound = inf
   90     CONTINUE
  100     CONTINUE

      ELSE IF ((3).EQ. (which)) THEN
C
C     Computing DF
C
          df = 5.0
          CALL stinvr(zero,inf,0.5,0.5,5.0,tol,tol)
          status = 0
          CALL invr(status,df,fx,qleft,qhi)
  110     IF (.NOT. (status.EQ.1)) GO TO 120
          fx = cum(t,df,p)
          CALL invr(status,df,fx,qleft,qhi)
          GO TO 110

  120     IF (.NOT. (status.EQ.-1)) GO TO 150
          IF (.NOT. (qleft)) GO TO 130
          status = 1
          bound = zero
          GO TO 140

  130     status = 2
          bound = inf
  140     CONTINUE
  150 END IF

      RETURN

      END
