      REAL FUNCTION cumchn(x,df,pnonc)
C
C**********************************************************************
C
C     REAL FUNCTION CUMCHN(X,DF,PNONC)
C             CUMulative of the Non-central CHi-square distribution
C
C
C                              Function
C
C
C     Calculates     the       cumulative      non-central    chi-square
C     distribution, i.e.,  the probability   that  a   random   variable
C     which    follows  the  non-central chi-square  distribution,  with
C     non-centrality  parameter    PNONC  and   continuous  degrees   of
C     freedom DF, is less than or equal to X.
C
C
C                              Arguments
C
C
C     X       --> Upper limit of integration of the non-central
C                 chi-square distribution.
C                                                 X is REAL
C
C     DF      --> Degrees of freedom of the non-central
C                 chi-square distribution.
C                                                 DF is REAL
C
C     PNONC   --> Non-centrality parameter of the non-central
C                 chi-square distribution.
C                                                 PNONC is REAL
C
C
C                              Method
C
C
C     Uses  formula  26.4.25   of  Abramowitz  and  Stegun, Handbook  of
C     Mathematical    Functions,  US   NBS   (1966)    to calculate  the
C     non-central chi-square.
C
C
C                              Variables
C
C
C     EPS     --- Convergence criterion.  The sum stops when a
C                 term is less than EPS*SUM.
C                                                 EPS is REAL
C
C     NTIRED  --- Maximum number of terms to be evaluated
C                 in each sum.
C                                                 NTIRED is INTEGER
C
C     QCONV   --- .TRUE. if convergence achieved -
C                 i.e., program did not stop on NTIRED criterion.
C                                                 QCONV is LOGICAL
C
C**********************************************************************
C
C
C     .. Scalar Arguments ..
      REAL df,pnonc,x
C     ..
C     .. Local Scalars ..
      REAL adj,centaj,centwt,chid2,dfd2,eps,lcntaj,lcntwt,lfact,pcent,
     +     pterm,sum,sumadj,term,wt,xnonc,xx
      INTEGER i,icent,iterb,iterf,ntired
C     ..
C     .. External Functions ..
      REAL alngam,cumchi
      EXTERNAL alngam,cumchi
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC alog,exp,float
C     ..
C     .. Statement Functions ..
      REAL dg
      LOGICAL qsmall,qtired
C     ..
C     .. Data statements ..
      DATA ntired,eps/1000,1.0E-5/
C     ..
C     .. Statement Function definitions ..
      qtired(i) = i .GT. ntired
      qsmall(xx) = sum .LT. 1.0E-20 .OR. xx .LT. eps*sum
      dg(i) = df + 2.0*float(i)
C     ..
C     .. Executable Statements ..
C
C
C
C     When non-centrality parameter is (essentially) zero,
C     use cumulative chi-square distribution
C
C
      IF (.NOT. (pnonc.LE.1.0E-10)) GO TO 10
      cumchn = cumchi(x,df)
      RETURN

   10 xnonc = pnonc/2.0
C**********************************************************************
C
C     The following code calcualtes the weight, chi-square, and
C     adjustment term for the central term in the infinite series.
C     The central term is the one in which the poisson weight is
C     greatest.  The adjustment term is the amount that must
C     be subtracted from the chi-square to move up two degrees
C     of freedom.
C
C**********************************************************************
      icent = xnonc
      IF (icent.EQ.0) icent = 1
      chid2 = x/2.0
C
C
C     Calculate central weight term
C
C
      lfact = alngam(float(icent+1))
      lcntwt = -xnonc + icent*alog(xnonc) - lfact
      centwt = exp(lcntwt)
C
C
C     Calculate central chi-square
C
C
      pcent = cumchi(x,dg(icent))
C
C
C     Calculate central adjustment term
C
C
      dfd2 = dg(icent)/2.0
      lfact = alngam(1.0+dfd2)
      lcntaj = dfd2*alog(chid2) - chid2 - lfact
      centaj = exp(lcntaj)
      sum = centwt*pcent
C**********************************************************************
C
C     Sum backwards from the central term towards zero.
C     Quit whenever either
C     (1) the zero term is reached, or
C     (2) the term gets small relative to the sum, or
C     (3) More than NTIRED terms are totaled.
C
C**********************************************************************
      iterb = 0
      sumadj = 0.0
      adj = centaj
      wt = centwt
      i = icent
C
      GO TO 30

   20 IF (qtired(iterb) .OR. qsmall(term) .OR. i.EQ.0) GO TO 40
   30 dfd2 = dg(i)/2.0
C
C
C     Adjust chi-square for two fewer degrees of freedom.
C     The adjusted value ends up in PTERM.
C
C
      adj = adj*dfd2/chid2
      sumadj = sumadj + adj
      pterm = pcent + sumadj
C
C
C     Adjust poisson weight for J decreased by one
C
C
      wt = wt* (i/xnonc)
      term = wt*pterm
      sum = sum + term
      i = i - 1
      iterb = iterb + 1
      GO TO 20

   40 iterf = 0
C**********************************************************************
C
C     Now sum forward from the central term towards infinity.
C     Quit when either
C     (1) the term gets small relative to the sum, or
C     (2) More than NTIRED terms are totaled.
C
C**********************************************************************
      sumadj = centaj
      adj = centaj
      wt = centwt
      i = icent
C
      GO TO 60

   50 IF (qtired(iterf) .OR. qsmall(term)) GO TO 70
C
C
C     Update weights for next higher J
C
C
   60 wt = wt* (xnonc/ (i+1))
C
C
C     Calculate PTERM and add term to sum
C
C
      pterm = pcent - sumadj
      term = wt*pterm
      sum = sum + term
C
C
C     Update adjustment term for DF for next iteration
C
C
      i = i + 1
      dfd2 = dg(i)/2.0
      adj = adj*chid2/dfd2
      sumadj = sumadj + adj
      iterf = iterf + 1
      GO TO 50

   70 cumchn = sum
C
      RETURN

      END
