      REAL FUNCTION cumhyp(k,n,m,l)
C**********************************************************************
C
C     REAL FUNCTION CUMHYP(K,N,M,L)
C             CUMulative HYPergeometric distribution
C
C
C                              Function
C
C
C     Calculates the  cumulative    hypergeometric distribution,   i.e.,
C     the  probability  that  a  random  variable   which  follows   the
C     hypergeometric distribution   with  sample  size N,   lot size   L
C     and M defectives in the lot, is less than or equal to K.
C
C
C                              Arguments
C
C
C     K       --> Argument for which hypergeometric distribution
C                 is to be evaluated.
C                                                 K is INTEGER
C
C     N       --> Sample size.
C                                                 N is INTEGER
C
C     M       --> Number of defectives in the lot.
C                                                 M is INTEGER
C
C     L       --> Lot size (must be greater than or equal to
C                 both N and M).
C                                                 L is INTEGER
C
C
C                              Method
C
C
C     probability  that   a hypergeometric  random  variable X   has the
C     value J+1  is  equal to the  probability that   X has the value  J
C     times the term
C
C             ((M-J) * (N-J)) / ((J+1) * (L-M-N+J+1))
C
C     The probability that  the random  variable   X is equal   to I  is
C     calculated conventionally as
C
C         C(M,I) * C(L-M,N-I) / C(L,N)
C
C     where  C(A,B)   is the  number  of   combinations  of   A  objects
C     taken  B  at a time  and I is the minimum  value   of J such  that
C     both binomial coefficients are defined in the numerator.
C
C     The   contributions  of   the    remaining values   of      J  are
C     accumulated  by  making  use  of  the  property    described above
C     for J=I+1,K.
C
C
C                              Variables
C
C
C     BC1     --- Temporary variable used to hold the log of one of the
C                 binomial coefficients used to calculate the first
C                 term of the cumulative hypergeometric.
C                                                 BC1 is REAL
C
C     BC2     --- Temporary variable used to hold the log of one of the
C                 binomial coefficients used to calculate the first
C                 term of the cumulative hypergeometric.
C                                                 BC2 is REAL
C
C     BC3     --- Temporary variable used to hold the log of one of the
C                 binomial coefficients used to calculate the first
C                 term of the cumulative hypergeometric.
C                                                 BC3 is REAL
C
C     T       --- Temporary variable used to hold cumulative
C                 hypergeometric at each term.
C                                                 T is REAL
C
C     J       --- Value of hypergeometric random variable at each
C                 iteration.
C                                                 J is INTEGER
C
C     I       --- Minimum value of J such that both binomial
C                 coefficients are defined in the numerator of the
C                 expression for the first term in the hypergeometric
C                 calculation. I is the maximum of 0 and M-L+N.
C                                                 I is INTEGER
C
C     MAXJ    --- Maximum value of J in calculation. If MAXJ is
C                 greater than the minimum of M and N, this function
C                 returns a probability of 1.
C                                                 MAXJ is INTEGER
C
C     LMN     --- L - M - N
C                 Used to avoid recalculation in loop.
C                                                  LMN is INTEGER
C
C
C                        Functions and Subroutines
C
C
C     LBICO  --- Function that calculates the log of the binomial
C                coefficient for N outcomes and S successes.
C                                        LBICO is REAL
C
C**********************************************************************
C     .. Scalar Arguments ..
      INTEGER k,l,m,n
C     ..
C     .. Local Scalars ..
      REAL bc1,bc2,bc3
      Real*8 t,cum ! BFL, 4-17-96
      INTEGER i,i99993,i99996,j,lmn,maxj
C     ..
C     .. External Functions ..
      REAL lbico
      EXTERNAL lbico
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC exp,max,min,real
C     ..
C     .. Executable Statements ..


C
C     Calculate minimum and maximum values which random variable
C     may take.
C
      i = max(0,m-l+n)
      maxj = min(n,m)
C
C     If K is out of valid bounds for J, return 0 or 1.
C     Otherwise, proceed with calculation.
C
      IF (.NOT. (k.LT.i)) GO TO 10
      !STOP 'K less than I in CUMHYP'	BFL, 3-18-96
      !write(*,*)  'K less than I in CUMHYP'

      GO TO 40

   10 IF (.NOT. (k.GT.maxj)) GO TO 20
      !STOP 'K greater than MAXJ in CUMHYP'
      !write(*,*) 'K greater than MAXJ in CUMHYP'

      GO TO 40
C     CALCULATE-HYPERGEOMETRIC
   20 ASSIGN 30 TO i99996
      GO TO 50

   30 CONTINUE
   40 RETURN
C
C
C
      !STOP '*** EXECUTION FLOWING INTO FLECS PROCEDURES ***'
      !write(*,*) '*** EXECUTION FLOWING INTO FLECS PROCEDURES ***'
C     TO CALCULATE-HYPERGEOMETRIC
C**********************************************************************
C
C     Calculate I'th term by method of binomial coefficients,
C     then calculate contributions of remaining terms recursively
C     by multiplication.
C
C**********************************************************************
   50 bc1 = lbico(m,i)
      bc2 = lbico(l-m,n-i)
      bc3 = lbico(l,n)
C
      !write(*,*) bc1+bc2-bc3
      t = dexp(bc1+ (bc2-bc3))
      cum = t
      !write(*,*) 't,cum ',t,cum
      !read(*,*)
C
      IF (t.LT.0.0) GO TO 70
C     CALC-CONTRIB-REMAIN-TERMS
      ASSIGN 60 TO i99993
      GO TO 80
C
   60 CONTINUE
   70 cumhyp = cum
      GO TO i99996
C     TO CALC-CONTRIB-REMAIN-TERMS
C
C
C**********************************************************************
C
C     Calculate contributions of remaining terms recursively
C     by multiplication.
C
C**********************************************************************
   80 lmn = l - m - n
C
      j = i + 1
      GO TO 100

   90 j = j + 1
  100 IF (j.GT.k) GO TO 110

      !write(*,*) real(((m-j+1)* (n-j+1)))/real((j* (lmn+j)))
      t = t*real(((m-j+1)* (n-j+1)))/real((j* (lmn+j)))
      cum = cum + t
      !write(*,*) 't, cum below ',t,cum
      GO TO 90

  110 GO TO i99993
C
      END
