      LOGICAL FUNCTION qrzero(f,y,xlo,xhi,abstol,reltol)
C**********************************************************************
C
C     LOGICAL FUNCTION QRZERO(F,Y,XLO,XHI,ABSTOL,RELTOL)
C          QR ZERO of a bounded monotone function F - Y
C
C
C                              Function
C
C
C          Concise Description - Given a function F
C     find XLO such that F(XLO) = Y.
C
C          More Precise Description -
C
C     Input condition. F is a Real*8 function of a single
C     Real*8 argument and XLO and XHI are such that
C          (F(XLO)-Y)*(F(XHI)-Y)  .LE.  0.0
C
C     If the input condition is met, QRZERO returns .TRUE.
C     and output values of XLO and XHI satisfy the following
C          (F(XLO)-Y)*(F(XHI)-Y)  .LE. 0.
C          ABS(F(XLO)-Y)  .LE. ABS(F(XHI)-Y)
C          ABS(XLO-XHI)  .LE. TOL(X)
C     where
C          TOL(X) = MAX(ABSTOL,RELTOL*ABS(X))
C
C     If this algorithm does not find XLO and XHI satisfying
C     these conditions then QRZERO returns .FALSE.  This
C     implies that the input condition was not met.
C
C
C                              Arguments
C
C
C     F --> A function of a single Real*8 argument.
C          F must be declared EXTERNAL in the calling program.
C                         F is EXTERNAL
C
C     Y --> The equation to be solved is F(X) = Y.
C		     Y is Real*8
C
C     XLO <--> On input, the left endpoint of the interval to be
C           searched for a solution.  On output, an estimate of
C           an improved interval (see Function).
C		     XLO is Real*8
C
C     XHI <--> On input, the right endpoint of the interval to be
C           for a solution.  On output, an estimate of an improved
C           interval.
C		     XHI is Real*8
C
C     ABSTOL, RELTOL --> Two numbers that determine the accuracy
C                      of the solution.  See function for a
C                      precise definition.
C		     ABSTOL is Real*8
C		     RELTOL is Real*8
C
C
C                              Method
C
C
C          Algorithm R of the paper 'Two Efficient Algorithms with
C     Guaranteed Convergence for Finding a Zero of a Function'
C     by J. C. P. Bus and T. J. Dekker in ACM Transactions on
C     Mathematical Software, Volume 1, no. 4 page 330
C     (Dec. '75) is employed to find the zero of F(X)-Y.
C
C**********************************************************************
C     ..
C     .. Scalar Arguments ..
      Real*8 y,xlo,xhi,abstol,reltol
C     ..
C     .. Function Arguments ..
      Real*8 f
      EXTERNAL f
C     ..
C     .. Local Scalars ..
      Real*8 a,b,c,d,fa,fb,fc,fd,fda,fdb,m,mb,p,q,tol,w,zx
      INTEGER ext
      LOGICAL first
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC abs,max,sign
C     ..
C     .. Statement Functions ..
      Real*8 ftol
C     ..
C     .. Save statement ..
      SAVE
C     ..
C     .. Statement Function definitions ..
      ftol(zx) = 0.5*max(abstol,reltol*abs(zx))
C     ..
C     .. Executable Statements ..
      b = xlo
      fb = f(xlo) - y
      xlo = xhi
      a = xlo
      fa = f(xlo) - y
      first = .TRUE.
   10 c = a
      fc = fa
      ext = 0
   20 IF (.NOT. (abs(fc).LT.abs(fb))) GO TO 40
      IF (.NOT. (c.NE.a)) GO TO 30
      d = a
      fd = fa
   30 a = b
      fa = fb
      xlo = c
      b = xlo
      fb = fc
      c = a
      fc = fa
   40 tol = ftol(xlo)
      m = (c+b)*.5
      mb = m - b
      IF (.NOT. (abs(mb).GT.tol)) GO TO 170
      IF (.NOT. (ext.GT.3)) GO TO 50
      w = mb
      GO TO 130

   50 tol = sign(tol,mb)
      p = (b-a)*fb
      IF (.NOT. (first)) GO TO 60
      q = fa - fb
      first = .FALSE.
      GO TO 70

   60 fdb = (fd-fb)/ (d-b)
      fda = (fd-fa)/ (d-a)
      p = fda*p
      q = fdb*fa - fda*fb
   70 IF (.NOT. (p.LT.0.0)) GO TO 80
      p = -p
      q = -q
   80 IF (ext.EQ.3) p = p*2.0
      IF (.NOT. ((p*1.0).EQ.0.0.OR.p.LE. (q*tol))) GO TO 90
      w = tol
      GO TO 120

   90 IF (.NOT. (p.LT. (mb*q))) GO TO 100
      w = p/q
      GO TO 110

  100 w = mb
  110 CONTINUE
  120 CONTINUE
  130 d = a
      fd = fa
      a = b
      fa = fb
      b = b + w
      xlo = b
      fb = f(xlo) - y
      IF (.NOT. ((fc*fb).GE.0.0)) GO TO 140
      GO TO 10

  140 IF (.NOT. (w.EQ.mb)) GO TO 150
      ext = 0
      GO TO 160

  150 ext = ext + 1
  160 GO TO 20

  170 xhi = c
      qrzero = (fc.GE.0.0 .AND. fb.LE.0.0) .OR.
     +         (fc.LT.0.0 .AND. fb.GE.0.0)
      RETURN

      END
